/******************************************************************************

 *     TERMOMETRO -50 a +100 C
 *
 *     C/ PIC 16f818  visualizando em 3 displays de 7 seguimentos
 *     Utiliza sensor LM35
 *
 *     Autor: Cludio Larios
 *
 *     Data: 04/09/2015
 *
 *
 *     Indicar temperaturas na faixa de -50 at 100 Centigrados
 *     Resoluo ps virgula de 0,5 graus.
 *     Escala negativa sem ps virgula (uso do sinal '-')
 *     Ajuste Voltagem de referncia no pino 2 para 2,5 Volts
 *     Material didtico apenas.
 *     Use compilador CCS C
 *
 *     Arquivo integrante do blog Lrios.Tecnologia.Ws

 ******************************************************************************/

/* Escolha aqui o tipo de display (catodo ou anodo comum */

 //  #define ANODO_COMUM

//Descomente para anodo comum  e comente para catodo comum
//==============================================================================

#include <16F818.h>
#device  adc=10
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOPUT, NOPROTECT, NOBROWNOUT, MCLR, NOLVP, NOCPD,CCPB2
#use fast_io(a)
#use fast_io(b)
#ZERO_RAM

#byte PORTA  = 0X5
#byte PORTB  = 0X6
#byte ADRESH = 0X1E
#byte ADRESL = 0X9E
#byte ADCON0 = 0X1F
#byte ADCON1 = 0X9F
#byte OSCCON = 0X8F
#byte TRISA  = 0X85
#byte TRISB  = 0X86



#byte OPTION_REG = 0X81
#byte CCP1CON = 0X17

#bit GO_DONE = 0X1F.2
#bit SSPEN   = 0X14.5
#bit ADON    = 0X1F.0

//defines

#bit   k0  =     0X5.2  //catodo/anodo 0  pino 1
#bit   k1  =     0X5.6  //catodo/anodo 1  pino 15
#bit   k2  =     0X5.7  //catodo/anodo 1  pino 16
#bit  ponto =   0X6.7  //seguimento 'p' pino 13

#define  fator 5 // vref=2,5volts x 2
#define N_AMOSTRA   1000 //amostras por apresentao

//variveis globais
int1 f_ok,f_sel,f_neg,f_err;
int8 vez,dig0,dig1,dig2,dig0a,dig1a,dig2a;
int16 cnt,conv;
int32 RES,RES1;


//constantes para converso 7 seguimentos

//                         PGFEDCBA   SEGUIMENTOS
//                   PORTB 76543210
int8 const num[] = {     0B00111111,//0
                         0B00000110,//1
                         0B01011011,//2
                         0B01001111,//3
                         0B01100110,//4
                         0B01101101,//5
                         0B01111101,//6
                         0B00000111,//7
                         0B01111111,//8
                         0B01101111,//9
                         0B01000000, //10 '-' trao
                         0B00000000, //11' ' apaga
                         0B11111001  //12'E' erro
                                        };



void  multiplexa(){
    int8 a;
    if(f_ok) {dig0=dig0a; dig1=dig1a;dig2=dig2a; f_ok=0;}
    if(++vez>2)vez=0;
     PORTA=0;//desliga display

   switch (vez){
     case 0:{a= num[dig0];break;}//case 0:
     case 1:{a= num[dig1];break;}//case 1:
     case 2:{a= num[dig2];break;}//case 2:
    }//switch (vez)
      #ifdef ANODO_COMUM
         PORTB=~a;
       #else
         PORTB=a;
       #endif
        switch (vez){
        case 0: {k0=1;break;}
        case 1: {
                 if(!f_neg)
        #ifdef  ANODO_COMUM
             ponto=0;
        #else
             ponto=1;
        #endif
                 k1=1;break;}//liga display 'dezena'
        case 2: {k2=1;break;}//liga display 'centena'
         }


  delay_ms(2);
 } //void  multiplexa()



void delay_ad(){delay_us(10);}

 void le_lm35() {

  if(GO_DONE) return; //utiliza o timer 0 para temporizar numero de comverso


  if(!f_sel){
   RES+=(int16) ADRESH<<8;
   RES+=ADRESL; //obtem valor
   f_sel=1;
   ADCON0=0b11001001; //canal 1/ liga conversor
   delay_ad();

  }
  else{
   RES1+=(int16)ADRESH<<8;
   RES1+=ADRESL; //obtem valor
   f_sel=0;
   ADCON0=0b11000001; //canal 0/ liga conversor
   delay_ad();

  }
   if(++cnt<N_AMOSTRA){
    GO_DONE=1; //inicia nova converso
    return;
   }

  if(RES>=RES1){
   RES=RES-RES1;
   f_neg=0;
  }
  else{
     RES=RES1-RES;
     f_neg=1;
  }

   conv=RES/N_AMOSTRA;
   conv=conv* fator ;
   if(conv>999){f_err=1;f_neg=0;}

   cnt=0;

   dig2a= conv /100;//digito das centenas
   dig1a= (conv %100) /10;//digito das dezenas
   dig0a=(conv %100) %10;//digito da unidade

   if(f_neg){
       dig0a=dig1a;
       dig1a=dig2a;
       dig2a=10;   //trao
       if(dig1a==0)dig1a=11;//apaga
   }
   else{
       if(dig2a==0)dig2a=11; //apaga
   }

   if(f_err){
       dig2a=12; //'E'
       dig1a=12;//'E'
       dig0a=12;//'E'
       f_err=0;
   }

   f_sel=0;
   f_ok=1;
   RES=0;
   RES1=0;
   GO_DONE=1; //inicia nova converso


 }



/*****************************************************************************
                     Rotina Main
 ******************************************************************************/

void main(){
   OPTION_REG=0; //sem prescaller
   CCP1CON=0;
   SSPEN=0;//DESLIGA SSP
   OSCCON=0B01100100; //escolhe 4MHZ interno
   ADCON0=0b11000001; //canal 0/ liga conversor
   ADCON1=0b11000101; //frequencia para oscilador interno
   PORTA=0;//desliga display
   TRISA= 0B00111011;//RA7,6 e 2 saidas
   TRISB=0;//seguimentos
   dig0=8;//teste de digitos
   dig1=8;//idem
   dig2=8;//idem
   cnt=0;
   RES=0;
   RES1=0;
   f_sel=0;
   f_err=0;
   ADON=1; //liga a/d


 /*****************************************************************************
               Lao Principal de Repetio
 ******************************************************************************/

 for(;;){
     le_lm35();//le a tenso gerada pelo LM35
     multiplexa();//apresentao nos displays
  }
}
