/******************************************************************************
 
 *     TERMOMETRO -50 a +100 C
 * 
 *     C/ PIC 16f818  visualizando em 3 displays de 7 seguimentos
 *     Utiliza sensor LM35
 *
 *     Autor: Cludio Larios
 * 
 *     Data: 04/09/2015
 * 
 * 
 *     Indicar temperaturas na faixa de -50 at 100 Centigrados
 *     Resoluo ps virgula de 0,5 graus.
 *     Escala negativa sem ps virgula (uso do sinal '-')
 *     Ajuste Voltagem de referncia no pino 2 para 2,5 Volts
 *     Material didtico apenas.
 *     Use compilador XC8 da Microchip
 *     
 *     Arquivo integrante do blog Lrios.Tecnologia.Ws 
 
 ******************************************************************************/

/* Escolha aqui o tipo de display (catodo ou anodo comum */

   #define ANODO_COMUM

//Descomente para anodo comum  e comente para catodo comum
//==============================================================================

#include <xc.h>

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG
#pragma config FOSC = INTOSCIO  // Oscillator Selection bits (INTRC oscillator; port I/O function on both RA6/OSC2/CLKO pin and RA7/OSC1/CLKI pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = ON       // RA5/MCLR/VPP Pin Function Select bit (RA5/MCLR/VPP pin function is MCLR)
#pragma config BOREN = OFF      // Brown-out Reset Enable bit (BOR disabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable bit (RB3/PGM pin has digital I/O function, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EE Memory Code Protection bit (Code protection off)
#pragma config WRT = OFF        // Flash Program Memory Write Enable bits (Write protection off)
#pragma config CCPMX = RB3      // CCP1 Pin Selection bit (CCP1 function on RB2)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)

//defines
#define _XTAL_FREQ  4000000   //frequncia de clock
#define   k0       RA2  //catodo/anodo 0  pino 1
#define   k1       RA6  //catodo/anodo 1  pino 15
#define   k2       RA7  //catodo/anodo 1  pino 16
#define   ponto    RB7  //seguimento 'p' pino 13
#define fator 5 // vref=2,5volts x 2
#define N_AMOSTRA   1000 //amostras por apresentao

//variveis globais
bit f_ok,f_sel,f_neg,f_err;
char vez,dig0,dig1,dig2,dig0a,dig1a,dig2a;
int conv,cnt;
long RES,RES1;


//constantes para converso 7 seguimentos

//                         PGFEDCBA   SEGUIMENTOS
//                   PORTB 76543210
char const num[] = {     0B00111111,//0
                         0B00000110,//1
                         0B01011011,//2
                         0B01001111,//3
                         0B01100110,//4
                         0B01101101,//5
                         0B01111101,//6
                         0B00000111,//7
                         0B01111111,//8
                         0B01101111,//9
                         0B01000000, //10 '-' trao
                         0B00000000, //11' ' apaga
                         0B11111001  //12'E' erro
                                        };



void  multiplexa(){
    char a;
    if(f_ok) {dig0=dig0a; dig1=dig1a;dig2=dig2a; f_ok=0;}
    if(++vez>2)vez=0;
     PORTA=0;//desliga display

   switch (vez){
     case 0:{a= num[dig0];break;}//case 0:
     case 1:{a= num[dig1];break;}//case 1:
     case 2:{a= num[dig2];break;}//case 2:
    }//switch (vez)
      #ifdef ANODO_COMUM
         PORTB=~a;
       #else
         PORTB=a;
       #endif
        switch (vez){
        case 0: {k0=1;break;}
        case 1: {
                 if(!f_neg)
        #ifdef  ANODO_COMUM
             ponto=0;
        #else
             ponto=1;
        #endif
                 k1=1;break;}//liga display 'dezena'
        case 2: {k2=1;break;}//liga display 'centena'
         }
    
      
   __delay_ms(2);
 } //void  multiplexa()



void delay_ad(){__delay_us(10);}
 
 void le_lm35() {

  if(GO_DONE) return; //utiliza o timer 0 para temporizar numero de comverso
  
  
  if(!f_sel){
   RES+=ADRESH<<8;
   RES+=ADRESL; //obtem valor
   f_sel=1;
   ADCON0=0b11001001; //canal 1/ liga conversor
   delay_ad();
   
  }
  else{
   RES1+=ADRESH<<8;
   RES1+=ADRESL; //obtem valor
   f_sel=0;
   ADCON0=0b11000001; //canal 0/ liga conversor
   delay_ad();
    
  }
   if(++cnt<N_AMOSTRA){
    GO_DONE=1; //inicia nova converso
    return;
   }
  
  if(RES>=RES1){
   RES=RES-RES1;
   f_neg=0;
  }
  else{
     RES=RES1-RES; 
     f_neg=1; 
  }
  
   conv=RES/N_AMOSTRA;
   conv=conv*fator;
   if(conv>999){f_err=1;f_neg=0;}

   cnt=0;
 
   dig2a= conv /100;//digito das centenas  
   dig1a= (conv %100) /10;//digito das dezenas
   dig0a=(conv %100) %10;//digito da unidade
   
   if(f_neg){
       dig0a=dig1a;
       dig1a=dig2a;
       dig2a=10;   //trao    
       if(dig1a==0)dig1a=11;//apaga
   }
   else{
       if(dig2a==0)dig2a=11; //apaga
   }
   
   if(f_err){
       dig2a=12; //'E'
       dig1a=12;//'E'
       dig0a=12;//'E'
       f_err=0;
   }
   
   f_sel=0;
   f_ok=1;
   RES=0;
   RES1=0;
   GO_DONE=1; //inicia nova converso


 }
 
 
 
/*****************************************************************************
                     Rotina Main
 ******************************************************************************/ 

void main(){
   OPTION_REG=0; //sem prescaller
   CCP1CON=0;
   SSPEN=0;//DESLIGA SSP
   OSCCON=0B01100100; //escolhe 4MHZ interno
   ADCON0=0b11000001; //canal 0/ liga conversor
   ADCON1=0b11000101; //frequencia para oscilador interno
   PORTA=0;//desliga display
   TRISA= 0B00111011;//RA7,6 e 2 saidas
   TRISB=0;//seguimentos 
   dig0=8;//teste de seguimentos
   dig1=8;//idem
   dig2=8;//idem
   cnt=0;
   RES=0;
   RES1=0;
   f_sel=0;
   f_err=0;
   ADON=1; //liga a/d


 /*****************************************************************************
               Lao Principal de Repetio
 ******************************************************************************/ 

 for(;;){
     le_lm35();//le a tenso gerada pelo LM35
     multiplexa();//apresentao nos displays
  }
}